import sys

import twisted.internet.endpoints
import twisted.internet.protocol
import twisted.protocols.amp
import twisted.python.log
import twisted.internet.defer

import altwistendpy.examples.amp.commands


def connect(reactor):
    endpoint = twisted.internet.endpoints.TCP4ClientEndpoint(
        reactor,
        "127.0.0.1",
        8750,
    )
    return endpoint.connect(
        twisted.internet.protocol.Factory.forProtocol(
            twisted.protocols.amp.AMP,
        ),
    )


@twisted.internet.defer.inlineCallbacks
def client(reactor, text):
    twisted.python.log.startLogging(sys.stdout)

    protocol = yield connect(reactor)
    result = yield protocol.callRemote(
        altwistendpy.examples.amp.commands.Print,
        text=text,
    )
    print('result: {}'.format(repr(result)))



import twisted.protocols.amp


class PrintError(Exception):
    pass


class Print(twisted.protocols.amp.Command):
    arguments = [
        (b'text', twisted.protocols.amp.Unicode()),
    ]

    response = [
        (b'success', twisted.protocols.amp.Boolean()),
    ]

    errors = {
        PrintError: b'print-error',
    }


class Commands(twisted.protocols.amp.CommandLocator):
    @Print.responder
    def print(self, text):
        print('printing: {}'.format(repr(text)))

        return {'success': True}



import attr
import click
import twisted.internet.defer

import altwistendpy.examples.amp.client
import altwistendpy.examples.amp.server


@attr.s
class ContextObject:
    reactor = attr.ib()


@click.group()
@click.pass_context
def cli(context):
    import twisted.internet.reactor

    twisted.internet.defer.setDebugging(True)

    context.obj = ContextObject(
        reactor=twisted.internet.reactor,
    )


@cli.command()
@click.pass_obj
def serve(context):
    altwistendpy.examples.amp.server.serve(reactor=context.reactor)
    context.reactor.run()


@cli.command()
@click.option('--text', required=True)
@click.pass_obj
def connect(context, text):
    d = altwistendpy.examples.amp.client.client(
        reactor=context.reactor,
        text=text,
    )
    d.addErrback(twisted.python.log.err, "Connection failed")
    d.addCallback(lambda _: context.reactor.stop())
    context.reactor.run()



import twisted.protocols.amp
import twisted.internet.protocol
import twisted.internet.endpoints

import altwistendpy.examples.amp.commands


def serve(reactor):
    endpoint = twisted.internet.endpoints.TCP4ServerEndpoint(reactor, 8750)
    factory = twisted.internet.protocol.Factory()
    factory.protocol = lambda: twisted.protocols.amp.AMP(
        locator=altwistendpy.examples.amp.commands.Commands(),
    )

    endpoint.listen(factory)



import twisted.internet.defer


# TODO: CAMPid copied from epyqlib, roughly
def sleep(seconds, reactor=None):
    if reactor is None:
        import twisted.internet.reactor
        reactor = twisted.internet.reactor

    d = twisted.internet.defer.Deferred()

    reactor.callLater(seconds, d.callback, None)

    return d



import setuptools

import versioneer


setuptools.setup(
    name='altwistendpy',
    description="Extras for working with Twisted.",
    author='Kyle Altendorf',
    author_email='sda@fstab.net',
    url='https://github.com/altendky/altwistendpy',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    entry_points={
        'console_scripts': [
            'amp = altwistendpy.examples.amp.cli:cli',
        ],
    },
    install_requires=[
        'attrs',
        'click',
        'gitignoreio',
        'twine',
        'twisted',
        'wheel',
    ],
)
